## Copyright 2024 Kakusui LLC (https://kakusui.org) (https://github.com/Kakusui) (https://github.com/Kakusui/Elucidate)
## Use of this source code is governed by an GNU Lesser General Public License v2.1
## license that can be found in the LICENSE file.

from .version import VERSION as __version__  # noqa

__author__ = "Kaden Bilyeu (Bikatr7) <Bikatr7@proton.me>"

from .elucidate import Elucidate

__all__ = ["Elucidate"]